@@ -1,4 +1,8 @@
-package DBIx::Class::CDBICompat::AttributeAPI;
+package # hide from PAUSE 
+    DBIx::Class::CDBICompat::AttributeAPI;
+
+use strict;
+use warnings;
 
 sub _attrs {
   my ($self, @atts) = @_;
