@@ -1,4 +1,8 @@
-package DBIx::Class::CDBICompat::GetSet;
+package # hide from PAUSE 
+    DBIx::Class::CDBICompat::GetSet;
+
+use strict;
+use warnings;
 
 #use base qw/Class::Accessor/;
 
