@@ -7,6 +7,7 @@ no warnings 'qw';
 use base qw/DBIx::Class/;
 
 __PACKAGE__->load_components(qw/
+  Serialize::Storable
   InflateColumn
   Relationship
   PK
@@ -48,10 +49,6 @@ The core modules currently are:
 
 =back
 
-If you are also using a L<DBIx::Class::PK::Auto> component, please
-make sure you load it correctly. Refer to
-L<DBIx::Class::PK::Auto/DESCRIPTION> for more information.
-
 =head1 AUTHORS
 
 Matt S. Trout <mst@shadowcatsystems.co.uk>
