@@ -1,5 +1,8 @@
 package DBIx::Class::DB;
 
+use strict;
+use warnings;
+
 use base qw/DBIx::Class/;
 use DBIx::Class::Schema;
 use DBIx::Class::Storage::DBI;
@@ -8,13 +11,16 @@ use DBI;
 
 __PACKAGE__->load_components(qw/ResultSetProxy/);
 
-*dbi_commit = \&txn_commit;
-*dbi_rollback = \&txn_rollback;
+{
+    no warnings 'once';
+    *dbi_commit = \&txn_commit;
+    *dbi_rollback = \&txn_rollback;
+}
 
 sub storage { shift->schema_instance(@_)->storage; }
 
 sub resultset_instance {
-  my $class = shift;
+  my $class = ref $_[0] || $_[0];
   my $source = $class->result_source_instance;
   if ($source->result_class ne $class) {
     $source = $source->new($source);
@@ -39,7 +45,8 @@ DBIx::Class::DB - Non-recommended classdata schema component
   package MyDB::MyTable;
 
   use base qw/MyDB/;
-  __PACKAGE__->load_components('Core'); # just load this in MyDB if it will always be there
+  __PACKAGE__->load_components('Core'); # just load this in MyDB if it will
+                                        # always be there
 
   ...
 
@@ -67,7 +74,7 @@ it. See resolve_class below.
 =cut
 
 __PACKAGE__->mk_classdata('class_resolver' =>
-                            'DBIx::Class::ClassResolver::PassThrough');
+                          'DBIx::Class::ClassResolver::PassThrough');
 
 =head2 connection
 
@@ -106,7 +113,7 @@ Begins a transaction (does nothing if AutoCommit is off).
 
 =cut
 
-sub txn_begin { $_[0]->schema_instance->txn_begin }
+sub txn_begin { shift->schema_instance->txn_begin(@_); }
 
 =head2 txn_commit
 
@@ -114,7 +121,7 @@ Commits the current transaction.
 
 =cut
 
-sub txn_commit { $_[0]->schema_instance->txn_commit }
+sub txn_commit { shift->schema_instance->txn_commit(@_); }
 
 =head2 txn_rollback
 
@@ -122,7 +129,17 @@ Rolls back the current transaction.
 
 =cut
 
-sub txn_rollback { $_[0]->schema_instance->txn_rollback }
+sub txn_rollback { shift->schema_instance->txn_rollback(@_); }
+
+=head2 txn_do
+
+Executes a block of code transactionally. If this code reference
+throws an exception, the transaction is rolled back and the exception
+is rethrown. See L<DBIx::Class::Schema/"txn_do"> for more details.
+
+=cut
+
+sub txn_do { shift->schema_instance->txn_do(@_); }
 
 {
   my $warn;
