@@ -1,4 +1,8 @@
-package DBIx::Class::Relationship::CascadeActions;
+package # hide from PAUSE
+    DBIx::Class::Relationship::CascadeActions;
+
+use strict;
+use warnings;
 
 sub delete {
   my ($self, @rest) = @_;
