@@ -31,10 +31,11 @@ table, and tie it to the class.
 =cut
 
 use strict;
+use warnings;
 
 use base qw/DBIx::Class/;
 
-__PACKAGE__->load_components(qw/PK::Auto::SQLite CDBICompat Core DB/);
+__PACKAGE__->load_components(qw/PK::Auto CDBICompat Core DB/);
 
 use File::Temp qw/tempfile/;
 my (undef, $DB) = tempfile();
@@ -70,7 +71,9 @@ sub _create_test_table {
         $class->sql__create_me($class->create_sql)->execute unless @vals;
 }
 
-=head2 create_sql (abstract)
+=head2 create_sql
+
+This is an abstract method you must override.
 
 	sub create_sql { 
 		return q{
